# module load R/3.6.1-foss-2015a-bare
# module load CDO/1.9.8-foss-2015a
# module load HDF5/1.8.14-foss-2015a

library(easyVerification)
library(startR)
library(s2dv)

seas5 <- "/esarchive/oper/S2S-downscale/seas5/tas_Nov_DJF-1981-2020.nc"
down <- "/esarchive/scratch/llledo/downscale/downscaled.nc"
era5 <- "/esarchive/oper/S2S-downscale/era5/tas_seas_1979-2018.nc"
uerra <- "/esarchive/oper/S2S-downscale/uerra/tas_seas_1979-2018.nc"

#hcst <- Start(dat = down,
hcst <- Start(dat = seas5,
              var = 'tas', 
              latitude ='all', 
              longitude='all', 
              ensemble='all', 
              time='all', 
              sdate='all',
              longitude_reorder = CircularSort(-180, 180),
              latitude_reorder = Sort(),
              longitude_var = "longitude",
              latitude_var = "latitude",
              num_procs = 1, retrieve = T,
              synonims = list(longitude = c('lon', 'longitude'), 
                              latitude=c('lat', 'latitude')))

lons <- as.vector(attributes(hcst)$Variables$common$longitude)
lats <- as.vector(attributes(hcst)$Variables$common$latitude)

#obs <- Start(dat = uerra,
obs <- Start(dat = era5,
              var = 'tas', 
	      #latitude ='all', 
	      #longitude='all', 
	      longitude = values(list(-10, 4)),
	      latitude = values(list(36, 44)),
	      time='all', 
	      longitude_reorder = CircularSort(-180, 180),
	      latitude_reorder = Sort(),
	      longitude_var = "longitude",
	      latitude_var = "latitude",
	      num_procs = 1, retrieve = T,
	      synonims = list(longitude = c('lon', 'longitude'), latitude=c('lat', 'latitude')),
		transform = CDORemapper,
		transform_extra_cells = 2,
		transform_params = list(grid = 'r360x181',
					method = 'conservative',
					crop = c(-10, 4, 36, 44)),
		transform_vars = c('latitude', 'longitude'),
		return_vars = list(	time = NULL,
					latitude = 'dat',
					longitude = 'dat')
	)
#lons2 <- as.vector(attributes(obs)$Variables$common$longitude)
#lats2 <- as.vector(attributes(obs)$Variables$common$latitude)
lons2 <- as.vector(attributes(obs)$Variables$dat1$longitude)
lats2 <- as.vector(attributes(obs)$Variables$dat1$latitude)

# seasonal averages
hcst <- drop(MeanDims(hcst, "time"))
hyears <- 1981:2020

obs <- drop(obs)
obs.djf <- obs[,,1:161 %% 4 == 1]
oyears <- 1978:2018 # indicates December year
obs.djf <- obs.djf[,,2:40] # discard JF 1979 and D 2018, incomplete seasons
oyears <- oyears[2:40]

verif.years <- intersect(hyears,oyears)

o <- obs.djf[,,oyears %in% verif.years]
f <- hcst[,,,hyears %in% verif.years]

# subset region
#idx.lon = lons2 %in% lons
#idx.lat = lats2 %in% lats
#o <- o[idx.lat,idx.lon,]

#o <- aperm(o, c(3,1,2))
#f <- aperm(f, c(3,4,1,2))
o <- aperm(o, c(3,2,1))
f <- aperm(f, c(3,4,1,2))

# skill scores
rpss <- veriApply("EnsRpss", fcst=f, obs=o, prob= 1:2 / 3, ensdim=1, tdim=2, na.rm=T)
corr <- veriApply("EnsCorr", fcst=f, obs=o, ensdim=1, tdim=2, na.rm=T)

PlotEquiMap(rpss$skillscore,lons,lats,filled.continents=F, toptitle="tas", units="RPSS", brks=seq(-0.2,0.2,0.02))
PlotEquiMap(corr,lons,lats,filled.continents=F, toptitle="tas", units="Correlation", brks=seq(-0.5,0.5,0.05))
